/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(615bc23d-6346-4b15-9c10-add002f140b6)]
interface nsIDisplayInfo : nsISupports
{
  readonly attribute long id;
  readonly attribute boolean connected;
};
