/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * vim: sw=2 ts=2 sts=2 expandtab 
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(00da7d20-3768-4398-bedc-e310c324b3f0)]
interface mozIStoragePendingStatement : nsISupports {

  /**
   * Cancels a pending statement, if possible.  This will only fail if you try
   * cancel more than once.
   *
   * @note For read statements (such as SELECT), you will no longer receive any
   *       notifications about results once cancel is called.
   */
   void cancel();
};
