/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMMouseEvent.idl"

[builtinclass, uuid(00404ebe-4598-4c4e-a60a-d7491922f9db)]
interface nsIDOMMouseScrollEvent : nsIDOMMouseEvent
{
  const long HORIZONTAL_AXIS = 1;
  const long VERTICAL_AXIS = 2;

  readonly attribute long axis;

  void                initMouseScrollEvent(in DOMString typeArg,
                                           in boolean canBubbleArg,
                                           in boolean cancelableArg,
                                           in nsIDOMWindow viewArg,
                                           in long detailArg,
                                           in long screenXArg,
                                           in long screenYArg,
                                           in long clientXArg,
                                           in long clientYArg,
                                           in boolean ctrlKeyArg,
                                           in boolean altKeyArg,
                                           in boolean shiftKeyArg,
                                           in boolean metaKeyArg,
                                           in unsigned short buttonArg,
                                           in nsIDOMEventTarget relatedTargetArg,
                                           in long axis);
};
