/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(f3fe3b48-fe81-4ec9-90ab-648ac9403466)]
interface nsIPaymentFlowInfo : nsISupports
{
  // Payment provider uri.
  attribute DOMString uri;

  // Base64 encoded and digitally signed payment request information.
  attribute DOMString jwt;

  // Payment providers expect the payment information as GET or POST
  // parameters.
  attribute DOMString requestMethod;

  // Payment provider name.
  attribute DOMString name;

  // Payment provider description.
  attribute DOMString description;
};
