/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"
#include "nsIDOMWindow.idl"

/**
 * The nsIDOMMessageEvent interface is used for server-sent events and for
 * cross-domain messaging.
 *
 * For more information on this interface, please see
 * http://www.whatwg.org/specs/web-apps/current-work/#messageevent
 */
[builtinclass, uuid(5d57bc56-30cf-4839-9e98-17f940120ec0)]
interface nsIDOMMessageEvent : nsISupports
{
  /**
   * Custom string data associated with this event.
   */
  [implicit_jscontext]
  readonly attribute jsval data;
  
  /**
   * The origin of the site from which this event originated, which is the
   * scheme, ":", and if the URI has a host, "//" followed by the
   * host, and if the port is not the default for the given scheme,
   * ":" followed by that port.  This value does not have a trailing slash.
   */
  readonly attribute DOMString origin;

  /**
   * The last event ID string of the event source, for server-sent DOM events; this
   * value is the empty string for cross-origin messaging.
   */
  readonly attribute DOMString lastEventId;
  
  /**
   * The window which originated this event.
   */
  readonly attribute nsIDOMWindow source;

  /**
   * Initializes this event with the given data, in a manner analogous to
   * the similarly-named method on the nsIDOMEvent interface, also setting the
   * data, origin, source, and lastEventId attributes of this appropriately.
   */
  void initMessageEvent(in DOMString aType,
                        in boolean aCanBubble,
                        in boolean aCancelable,
                        in jsval aData,
                        in DOMString aOrigin,
                        in DOMString aLastEventId,
                        in nsIDOMWindow aSource);
};
